###############################################################################
#
# Bash remediation role for profile standard
# Profile Title:  Standard System Security Profile
# Profile Description:
# This profile contains rules to ensure standard security baseline
# of Red Hat Enterprise Linux 7 system. Regardless of your system's workload
# all of these checks should pass.
#
# Benchmark ID:  RHEL-7
# Benchmark Version:  0.1.36
#
# XCCDF Version:  1.1
#
# This file was generated by OpenSCAP 1.2.16 using:
# 	$ oscap xccdf generate fix --profile standard --template urn:xccdf:fix:script:sh xccdf-file.xml 
#
# This script is generated from an OpenSCAP profile without preliminary evaluation.
# It attempts to fix every selected rule, even if the system is already compliant.
#
# How to apply this remediation role:
# $ sudo ./remediation-role.sh
#
###############################################################################

###############################################################################
# BEGIN fix (1 / 13) for 'ensure_redhat_gpgkey_installed'
###############################################################################
(>&2 echo "Remediating rule 1/13: 'ensure_redhat_gpgkey_installed'")
# The two fingerprints below are retrieved from https://access.redhat.com/security/team/key
readonly REDHAT_RELEASE_2_FINGERPRINT="567E 347A D004 4ADE 55BA 8A5F 199E 2F91 FD43 1D51"
readonly REDHAT_AUXILIARY_FINGERPRINT="43A6 E49C 4A38 F4BE 9ABF 2A53 4568 9C88 2FA6 58E0"
# Location of the key we would like to import (once it's integrity verified)
readonly REDHAT_RELEASE_KEY="/etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release"

RPM_GPG_DIR_PERMS=$(stat -c %a "$(dirname "$REDHAT_RELEASE_KEY")")

# Verify /etc/pki/rpm-gpg directory permissions are safe
if [ "${RPM_GPG_DIR_PERMS}" -le "755" ]
then
  # If they are safe, try to obtain fingerprints from the key file
  # (to ensure there won't be e.g. CRC error).
  IFS=$'\n' GPG_OUT=($(gpg --with-fingerprint "${REDHAT_RELEASE_KEY}" | grep 'Key fingerprint ='))
  GPG_RESULT=$?
  # No CRC error, safe to proceed
  if [ "${GPG_RESULT}" -eq "0" ]
  then
    tr -s ' ' <<< "${GPG_OUT}" | grep -vE "${REDHAT_RELEASE_2_FINGERPRINT}|${REDHAT_AUXILIARY_FINGERPRINT}" || {
      # If file doesn't contains any keys with unknown fingerprint, import it
      rpm --import "${REDHAT_RELEASE_KEY}"
    }
  fi
fi
# END fix for 'ensure_redhat_gpgkey_installed'

###############################################################################
# BEGIN fix (2 / 13) for 'ensure_gpgcheck_globally_activated'
###############################################################################
(>&2 echo "Remediating rule 2/13: 'ensure_gpgcheck_globally_activated'")
# Function to replace configuration setting in config file or add the configuration setting if
# it does not exist.
#
# Expects four arguments:
#
# config_file:		Configuration file that will be modified
# key:			Configuration option to change
# value:		Value of the configuration option to change
# cce:			The CCE identifier or '@CCENUM@' if no CCE identifier exists
#
# Optional arugments:
#
# format:		Optional argument to specify the format of how key/value should be
# 			modified/appended in the configuration file. The default is key = value.
#
# Example Call(s):
#
#     With default format of 'key = value':
#     replace_or_append '/etc/sysctl.conf' '^kernel.randomize_va_space' '2' '@CCENUM@'
#
#     With custom key/value format:
#     replace_or_append '/etc/sysconfig/selinux' '^SELINUX=' 'disabled' '@CCENUM@' '%s=%s'
#
#     With a variable:
#     replace_or_append '/etc/sysconfig/selinux' '^SELINUX=' $var_selinux_state '@CCENUM@' '%s=%s'
#
function replace_or_append {
  local config_file=$1
  local key=$2
  local value=$3
  local cce=$4
  local format=$5

  # Check sanity of the input
  if [ $# -lt "3" ]
  then
        echo "Usage: replace_or_append 'config_file_location' 'key_to_search' 'new_value'"
        echo
        echo "If symlinks need to be taken into account, add yes/no to the last argument"
        echo "to allow to 'follow_symlinks'."
        echo "Aborting."
        exit 1
  fi

  # Test if the config_file is a symbolic link. If so, use --follow-symlinks with sed.
  # Otherwise, regular sed command will do.
  if test -L $config_file; then
    sed_command="sed -i --follow-symlinks"
  else
    sed_command="sed -i"
  fi

  # Test that the cce arg is not empty or does not equal @CCENUM@.
  # If @CCENUM@ exists, it means that there is no CCE assigned.
  if ! [ "x$cce" = x ] && [ "$cce" != '@CCENUM@' ]; then
    cce="CCE-${cce}"
  else
    cce="CCE"
  fi

  # Strip any search characters in the key arg so that the key can be replaced without
  # adding any search characters to the config file.
  stripped_key=$(sed "s/[\^=\$,;+]*//g" <<< $key)

  # If there is no print format specified in the last arg, use the default format.
  if ! [ "x$format" = x ] ; then
    printf -v formatted_output "$format" "$stripped_key" "$value"
  else
    formatted_output="$stripped_key = $value"
  fi

  # If the key exists, change it. Otherwise, add it to the config_file.
  if `grep -qi "$key" $config_file` ; then
    eval '$sed_command "s/$key.*/$formatted_output/g" $config_file'
  else
    # \n is precaution for case where file ends without trailing newline
    echo -e "\n# Per $cce: Set $formatted_output in $config_file" >> $config_file
    echo -e "$formatted_output" >> $config_file
  fi

}

replace_or_append '/etc/yum.conf' '^gpgcheck' '1' 'CCE-26989-4'
# END fix for 'ensure_gpgcheck_globally_activated'

###############################################################################
# BEGIN fix (3 / 13) for 'security_patches_up_to_date'
###############################################################################
(>&2 echo "Remediating rule 3/13: 'security_patches_up_to_date'")
yum -y update
# END fix for 'security_patches_up_to_date'

###############################################################################
# BEGIN fix (4 / 13) for 'rpm_verify_permissions'
###############################################################################
(>&2 echo "Remediating rule 4/13: 'rpm_verify_permissions'")

# Declare array to hold list of RPM packages we need to correct permissions for
declare -a SETPERMS_RPM_LIST

# Create a list of files on the system having permissions different from what
# is expected by the RPM database
FILES_WITH_INCORRECT_PERMS=($(rpm -Va --nofiledigest | grep '^.M' | cut -d ' ' -f5-))

# For each file path from that list:
# * Determine the RPM package the file path is shipped by,
# * Include it into SETPERMS_RPM_LIST array

for FILE_PATH in "${FILES_WITH_INCORRECT_PERMS[@]}"
do
	RPM_PACKAGE=$(rpm -qf "$FILE_PATH")
	SETPERMS_RPM_LIST=("${SETPERMS_RPM_LIST[@]}" "$RPM_PACKAGE")
done

# Remove duplicate mention of same RPM in $SETPERMS_RPM_LIST (if any)
SETPERMS_RPM_LIST=( $(echo "${SETPERMS_RPM_LIST[@]}" | sort -n | uniq) )

# For each of the RPM packages left in the list -- reset its permissions to the
# correct values
for RPM_PACKAGE in "${SETPERMS_RPM_LIST[@]}"
do
	rpm --setperms "${RPM_PACKAGE}"
done
# END fix for 'rpm_verify_permissions'

###############################################################################
# BEGIN fix (5 / 13) for 'rpm_verify_hashes'
###############################################################################
(>&2 echo "Remediating rule 5/13: 'rpm_verify_hashes'")
# FIX FOR THIS RULE IS MISSING
# END fix for 'rpm_verify_hashes'

###############################################################################
# BEGIN fix (6 / 13) for 'mount_option_dev_shm_nodev'
###############################################################################
(>&2 echo "Remediating rule 6/13: 'mount_option_dev_shm_nodev'")
# FIX FOR THIS RULE IS MISSING
# END fix for 'mount_option_dev_shm_nodev'

###############################################################################
# BEGIN fix (7 / 13) for 'mount_option_dev_shm_nosuid'
###############################################################################
(>&2 echo "Remediating rule 7/13: 'mount_option_dev_shm_nosuid'")
# FIX FOR THIS RULE IS MISSING
# END fix for 'mount_option_dev_shm_nosuid'

###############################################################################
# BEGIN fix (8 / 13) for 'dir_perms_world_writable_sticky_bits'
###############################################################################
(>&2 echo "Remediating rule 8/13: 'dir_perms_world_writable_sticky_bits'")
# FIX FOR THIS RULE IS MISSING
# END fix for 'dir_perms_world_writable_sticky_bits'

###############################################################################
# BEGIN fix (9 / 13) for 'file_permissions_unauthorized_world_writable'
###############################################################################
(>&2 echo "Remediating rule 9/13: 'file_permissions_unauthorized_world_writable'")
# FIX FOR THIS RULE IS MISSING
# END fix for 'file_permissions_unauthorized_world_writable'

###############################################################################
# BEGIN fix (10 / 13) for 'file_permissions_unauthorized_sgid'
###############################################################################
(>&2 echo "Remediating rule 10/13: 'file_permissions_unauthorized_sgid'")
# FIX FOR THIS RULE IS MISSING
# END fix for 'file_permissions_unauthorized_sgid'

###############################################################################
# BEGIN fix (11 / 13) for 'file_permissions_unauthorized_suid'
###############################################################################
(>&2 echo "Remediating rule 11/13: 'file_permissions_unauthorized_suid'")
# FIX FOR THIS RULE IS MISSING
# END fix for 'file_permissions_unauthorized_suid'

###############################################################################
# BEGIN fix (12 / 13) for 'no_empty_passwords'
###############################################################################
(>&2 echo "Remediating rule 12/13: 'no_empty_passwords'")
sed --follow-symlinks -i 's/\<nullok\>//g' /etc/pam.d/system-auth
sed --follow-symlinks -i 's/\<nullok\>//g' /etc/pam.d/password-auth
# END fix for 'no_empty_passwords'

###############################################################################
# BEGIN fix (13 / 13) for 'accounts_root_path_dirs_no_write'
###############################################################################
(>&2 echo "Remediating rule 13/13: 'accounts_root_path_dirs_no_write'")
# FIX FOR THIS RULE IS MISSING
# END fix for 'accounts_root_path_dirs_no_write'

